import dotenv from 'dotenv';
import { Bus, FisAppActor, FisMessage } from "../interface/transport.interface";
import { ConnectionAdapter } from "./connector.base";
import { AdaptorTransmissionRole, ConnectionState, ReceiverConnectionAdapter as ReceiverConnectionAdapterInterface, TransportEvent, TransportMessage, TransportService } from "../interface/connector.interface";
import { BehaviorSubject, filter, map, Observable, Observer, Subject, Subscription, takeWhile } from 'rxjs';

dotenv.config();
/* This transport manager will be instantiating the necessary transport to deal with tranmission and receiving from different receivers
So how?: */
export class ReceiverConnectionAdapter extends ConnectionAdapter implements ReceiverConnectionAdapterInterface {
    event: Subject<TransportEvent> = new Subject()

    constructor(adapter: TransportService) {
        super()
        // logic here
        this.adaptorTransmissionRole = AdaptorTransmissionRole.Receiver
        this.connector = adapter
    }

    getMessageBus(bus: Bus): Observable<any> {
        return new Observable((observable: Observer<any>) => {
            if (bus == Bus.ResponseMessageBus) {
                const subscription: Subscription = this.event.pipe(
                    filter((message: TransportEvent) => message.event === 'New Message'),
                    // take message only specific for this adapter. Although that itself wouldn't be necessary, considerng everything goes through transportEvent. I guess it's for better management
                    filter((message:TransportEvent) => (message.data as TransportMessage).target == this.connectorProfile.id), 
                    takeWhile((message: TransportEvent) => {
                        const shouldTake = ((message.data as TransportMessage).payload as FisMessage).data !== 'Complete';
                        if (!shouldTake) {
                            observable.complete();  // Ensure the observer is completed
                        }
                        return shouldTake;
                    }),
                    map(message => (message.data as TransportMessage).payload as FisMessage)
                ).subscribe((message: FisMessage) => {
                    observable.next(message);
                });
    
                // Clean up on unsubscription
                return () => {
                    subscription.unsubscribe();
                };
            }
            if (bus == Bus.NotificationMessageBus) {
                /// logic here
            }
            if (bus == Bus.ErrorMessageBus) {
                /// logic here
            }
        });
    }
}


