import { BehaviorSubject, Observable, Subject } from "rxjs"
import { Bus, FisAppActor, ReceiverProfile, TransmissionEvent, TransmissionMessage, TransmissionProfile, TransmitterProfile } from "./transport.interface"
import { ConnectionAdapter } from "../connector/connector.base"


export type TYPE = {
    adapterProfile: AdapterProfile,
    transmitterProfile: TransmitterProfile,
    receiverProfile: ReceiverProfile,
    requestResponseProfile: TransmissionProfile
}

export interface AdapterProfile {
    id: string,
    adapter: ConnectionAdapter,
    connectionState: BehaviorSubject<ConnectionState>
}


export interface ConnectionManager {
    transportService: TransportService
    // to get notified on what's going on in Transmission Manager
    transmissionEvent: Subject<TransmissionEvent>
    transportEvent: Subject<TransportEvent>
    // list of connection
    transmissionSet: ConnectionSet[]
    // Called by transmission manager to have an instance of these adapters
    getTransmitterConnectionAdapter(): TransmitterConnectionAdapter
    getReceiverConnectionAdapter(): ReceiverConnectionAdapter
    getRequestResponseConnectionAdapter(transmitterAdapter: TransmitterConnectionAdapter, receiverConnectionAdapter: ReceiverConnectionAdapter): RequestResponseConnectionAdapter
}

export interface ConnectionAdaptorBase  {
    connector: TransportService // this one will refer to the actual tranpsort service like websocket and so on
    connectorProfile: AdapterProfile
    connectionStateBus: BehaviorSubject<ConnectionState>
    adaptorTransmissionRole: AdaptorTransmissionRole

    subscribeConnectionState(): Observable<ConnectionState>
    publishConnectionState(): void
    connect(): void
    disconnect(): void
    getMessageBus(bus: Bus): Observable<any>
    getInfo(): any
}


export interface TransmitterConnectionAdapter extends ConnectionAdaptorBase {
    emit(message: TransmissionMessage): void
    emitStream(message: TransmissionMessage): void
}

export interface ReceiverConnectionAdapter extends ConnectionAdaptorBase {
    event: Subject<TransportEvent>
    subscribeMessages(messageFilter: any): Observable<any>
}

export interface RequestResponseConnectionAdapter extends TransmitterConnectionAdapter, ReceiverConnectionAdapter {
    send(message: TransmissionMessage): Observable<TransportMessage>
}

export type ConnectionState = 'ONLINE' | 'OFFLINE'
export enum AdaptorTransmissionRole {
    Transmitter,
    Receiver,
    RequestResponse
}

export enum Transport {
    Websocket,
    Grpc,
    Http
}

// TO be used for transmission at the trasport level
export interface TransportMessage {
    id: string,
    dateCreated: Date,
    transport: Transport,
    target?: string,
    payload: any
}

export interface TransportEvent {
    id: string,
    event: 'Server Started' | 'New Client' | 'Client Disconnected' | 'Client Reconnected' | `Server Disconnected` | 'New Message' | `Notification` | `New Server` | `New Transport`,
    data: any
}
export interface ConnectionSet {
    // TBD
}

export interface TransportService {
    getInfo(): any
    emit(message: TransportMessage): void
    subscribe(): Observable<TransportEvent> //all messages and whatever event will go through this, easier to implemnet across different transport protocol
}


export interface Info {
    transport: Transport
}