import { MessageTransmissionTransmitter } from "./msg.transmission.transmitter";
import { MessageTransmissionReceiver } from "./msg.transmission.receiver";
import { ConnectionManager } from "../connector/connector.manager";
import { MessageTransmission, MessageTransmissionManager as MessageTransmissionManagerInterface, ReceiverProfile, TransmissionEvent, TransmissionProfile, TransmitterProfile } from "../interface/transport.interface";
import { v4 as uuidv4 } from 'uuid'
import { AdaptorTransmissionRole, RequestResponseConnectionAdapter } from "../interface/connector.interface";
import { MessageTransmissionRequestResponse } from "./msg.transmission.request-response";
import { TransmitterConnectionAdapter } from "../connector/connector.transmitter";
import { ReceiverConnectionAdapter } from "../connector/connector.receiver";
import { filter, Subject } from "rxjs";
/*OKay, the application don't give a shit whose request it comes from. That will be the responsibility of this manager in particular.
So, from the transmtiter perspective, it would be as if it's only coming from one source, but the demultiplexing will occur here 
so to speak. The plan, for one type of transport, this manager will make sure all of the connector instances travel through here.
But of course, the transmitter or fis app actor can always instaniate more transmitter and receiver if needed, and there will be 
options to cater for wanting to instante different types of transport services according to their needs. */
export class MessageTransmissionManager implements MessageTransmissionManagerInterface {
    // transmission: MessageTransmission[] = []
    connectionManager!: ConnectionManager
    messageTransmissionTransmitters: MessageTransmissionTransmitter[] = []
    messageTransmissionReceiver: MessageTransmissionReceiver[] = []
    messageTransmissionRequestResponse: MessageTransmissionRequestResponse[] = []
    transmissionEvent: Subject<TransmissionEvent> = new Subject()

    constructor() {
       // logic here
    }

    /* so there will be some changes here. will nto be assigning just one, but all of them dynamically to pour into this boy
    Transmitter only have to call this once. */
    getTransmissionInstance(): MessageTransmission {
        let transmitter: MessageTransmissionTransmitter = this.getTransmitter()
        let receiver: MessageTransmissionReceiver = this.getReceiver()
        let transmission: MessageTransmission = {
            id: uuidv4(),
            receiverId: transmitter.getInfo().id,
            transmitterId: receiver.getInfo().id,
            transmitter: transmitter,
            receiver: receiver,
            requestResponse: this.getRequestResponse(transmitter, receiver)
        }
        this.instantiateConnectionManager() // start an adapter
        // this.transmission.push(transmission)
        return transmission
    }

    private getTransmitter(): MessageTransmissionTransmitter {
        let transmitterProfile: TransmitterProfile = {
            id: uuidv4(),
            name: '', // for now make it empty. We will use the assigned uuid here
            dateCreated: new Date()
        }
        return new MessageTransmissionTransmitter(transmitterProfile, AdaptorTransmissionRole.Transmitter, this.transmissionEvent)
    }

    private getReceiver(): MessageTransmissionReceiver {
        let receiverProfile: ReceiverProfile = {
            id: uuidv4(),
            name: '', // for now make it empty. We will use the assigned uuid here
            dateCreated: new Date()
        }
        return new MessageTransmissionReceiver(receiverProfile, AdaptorTransmissionRole.Receiver, this.transmissionEvent)
    }

    private getRequestResponse(transmitterInstance: MessageTransmissionTransmitter, receiverInstance: MessageTransmissionReceiver): MessageTransmissionRequestResponse {
        return new MessageTransmissionRequestResponse(transmitterInstance, receiverInstance, AdaptorTransmissionRole.RequestResponse, this.transmissionEvent)
    }

    private instantiateConnectionManager(): void {
        this.connectionManager = new ConnectionManager(this.transmissionEvent)
    }

}