import { filter, map, Observable, Observer, PartialObserver, Subject, Subscriber, Subscription, takeWhile, Unsubscribable } from 'rxjs';
import { AdapterProfile, AdaptorTransmissionRole, TransportEvent, TransportMessage } from '../interface/connector.interface';
import { MessageTransmissionBase } from './msg.transmission.base';
import { Bus, FisMessage, MessageReceiver as MessageReceiverInterface, ReceiverProfile, TransmissionEvent, TransmissionMessage, TransmitterProfile } from '../interface/transport.interface'
import { ConnectionAdapter } from '../connector/connector.base';
import { v4 as uuidv4 } from 'uuid'
import { ReceiverConnectionAdapter } from '../connector/connector.receiver';

export class MessageTransmissionReceiver extends MessageTransmissionBase implements MessageReceiverInterface {
    receiverProfile!: ReceiverProfile;
    private incomingTransportMessage: Subject<TransportMessage> = new Subject()

    constructor(profile: ReceiverProfile, role: AdaptorTransmissionRole, event: Observable<TransmissionEvent>) {
        super();

        this.setReceiver(profile, role)
        this.handleEvent(event)
    }

    setReceiver(receiverProfile: ReceiverProfile, role: AdaptorTransmissionRole): void {
        this.receiverProfile = receiverProfile
        this.transmissionRole = role
    }

    getMessageBus(bus: Bus): Observable<any> {
        return new Observable((observable: Observer<any>) => {
            // logic here
            if (bus == Bus.ResponseMessageBus) {
                const subscription: Subscription = this.incomingTransportMessage.pipe(
                    filter((message: TransportMessage) => message. === `Incoming Message`),
                    map(message => message.data as TransportMessage)
                ).subscribe((message) => {
                    observable.next(message);
                });

                // Clean up on unsubscription
                return () => {
                    subscription.unsubscribe();
                };
            }
        })
    }

    private handleEvent(event: Observable<TransmissionEvent>): void {
        event.pipe(
            filter((obj: TransmissionEvent) => obj.event == `New Receiver`),
            map((obj: TransmissionEvent) => (obj.data as ReceiverConnectionAdapter))
        ).subscribe((adapter: ReceiverConnectionAdapter) => {
            this.adaptorsArray.push({ id: adapter.getInfo().id, adapter: adapter } as unknown as AdapterProfile)
            adapter.getMessageBus(Bus.GeneralBus).subscribe(this.incomingTransportMessage)
        })
    }
}
