import { MessageTransmissionBase } from "./msg.transmission.base";
import { FisMessage, MessageTransmitter as MessageTransmitterInterface, TransmissionEvent, TransmissionMessage, TransmitterProfile } from '../interface/transport.interface'
import { AdapterProfile, AdaptorTransmissionRole, ConnectionState, TransportMessage, TransportService } from "../interface/connector.interface";
import { v4 as uuidv4 } from 'uuid'
import { TransmitterConnectionAdapter } from "../connector/connector.transmitter";
import { BehaviorSubject, filter, map, Observable } from "rxjs";

/* Take in all the messages that needs to be transported, and divide them accordingly. So the connector instances will do just that
connectors or adapters will have their own identifier*/
export class MessageTransmissionTransmitter extends MessageTransmissionBase implements MessageTransmitterInterface {
    transmitterProfile!: TransmitterProfile;

    constructor(profile: TransmitterProfile, role: AdaptorTransmissionRole, event: Observable<TransmissionEvent>) {
        super()
        this.setTransmitter(profile, role)

        this.handleEvent(event)
    }

    setTransmitter(transmitterProfile: TransmitterProfile, role: AdaptorTransmissionRole): void {
        this.transmitterProfile = transmitterProfile
        this.transmissionRole = role
    }

    emit(message: TransmissionMessage): void {
        // logic here, emit based on message
        let adapter: AdapterProfile | undefined = this.adaptorsArray.find(obj => obj.id == message.receiverId)
        if(adapter) {
            (adapter.adapter as TransmitterConnectionAdapter).emit(message)
        }
    }

    private handleEvent(event: Observable<TransmissionEvent>): void {
        event.pipe(
            filter((obj: TransmissionEvent) => obj.event == `New Receiver`),
            map(obj => (obj as TransmissionEvent).data as TransmitterConnectionAdapter)
        ).subscribe((adapter: TransmitterConnectionAdapter) => {
            this.adaptorsArray.push({ id: adapter.getInfo().id, adapter: adapter, connectionState: new BehaviorSubject<ConnectionState>(`OFFLINE`) }) // refers to channel ID, which will be embedded in these messages that pass through
        })
    }
}